package com.moyoung.moyoung_ble_plugin;

import android.app.Activity;
import android.util.Log;

import com.moyoung.moyoung_ble_plugin.method.ScanMethodHandler;
import com.moyoung.moyoung_ble_plugin.event.ScanDeviceHandler;
import com.moyoung.moyoung_ble_plugin.event.ConnEveChlRegister;
import com.moyoung.moyoung_ble_plugin.method.ConnectionMethodHandler;

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;

import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_SCAN_DEVICE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.METHOD_SCAN_DEVICE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.METHOD_CONNECTION;

/**
 * MoyoungBlePlugin
 */
public class MoyoungBlePlugin implements FlutterPlugin, ActivityAware {
    
    private final static String TAG = MoyoungBlePlugin.class.getSimpleName();
    private Activity activity;
    private BinaryMessenger binaryMgr;

    private MethodChannel bleScanMethodChannel;
    private EventChannel bleScanEveChl;
    private ScanMethodHandler bleMethodHandler;

    private MethodChannel connMethodChannel;
    private ConnectionMethodHandler connectionMethodHandler;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        binaryMgr = binding.getBinaryMessenger();

        ScanDeviceHandler scanEventHandler = registerScanEveChl();
        registerBleScanMethodChl(scanEventHandler);

        registerConnMethodChl();
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        bleMethodHandler.unbindMethodChannel(bleScanMethodChannel, bleScanEveChl);
        connectionMethodHandler.unbindChannel(connMethodChannel);
    }

    private void registerBleScanMethodChl(ScanDeviceHandler scanEventHandler) {
        bleMethodHandler = new ScanMethodHandler(scanEventHandler);
        bleScanMethodChannel = new MethodChannel(binaryMgr, METHOD_SCAN_DEVICE);
        bleScanMethodChannel.setMethodCallHandler(bleMethodHandler);
    }

    private void registerConnMethodChl() {
        ConnEveChlRegister connEveChlRegister = new ConnEveChlRegister(binaryMgr);
        connEveChlRegister.register();

        connectionMethodHandler = new ConnectionMethodHandler(connEveChlRegister);
        connMethodChannel = new MethodChannel(binaryMgr, METHOD_CONNECTION);
        connMethodChannel.setMethodCallHandler(connectionMethodHandler);
    }

    private ScanDeviceHandler registerScanEveChl() {
        ScanDeviceHandler scanEventHandler = new ScanDeviceHandler();
        bleScanEveChl = new EventChannel(binaryMgr, EVENT_SCAN_DEVICE);
        bleScanEveChl.setStreamHandler(scanEventHandler);

        return scanEventHandler;
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();

        bleMethodHandler.bindActivity(activity);
        connectionMethodHandler.bindActivity(activity);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    }

    @Override
    public void onDetachedFromActivity() {
        activity = null;
    }

}
