package com.moyoung.moyoung_ble_plugin.event;

import android.app.Activity;

import com.moyoung.moyoung_ble_plugin.base.BaseConnectionEventChannelHandler;

import java.util.ArrayList;
import java.util.List;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;

import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_BATTERY_SAVING;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_BLOOD_OXYGEN;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_BLOOD_PRESSURE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CALENDAR_EVENT;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CALL_NUMBER;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CAMERA;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CONTACT;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_DEVICE_BATTERY;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_DEVICE_RSSI;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_FIND_PHONE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_GPS_CHANGE_EVENT;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_HEART_RATE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_NEW_HRV;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_SOS_CHANGE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_STRESS;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_TRAINING_STATE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_PHONE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_SLEEP_CHANGE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CONNECTION_STATE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_STEPS_DETAIL;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_STEPS_CHANGE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_TEMP_CHANGE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_TRAIN;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_WEATHER_CHANGE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_CONTACT_AVATAR;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_ECG;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_FILE_TRANS;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_FIRMWARE_UPGRADE;
import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_WF_FILE_TRANS;

public class ConnEveChlRegister {
    private final static String TAG = ConnEveChlRegister.class.getSimpleName();

    private final List<EventChannel> eveChlList = new ArrayList<>();
    public final List<BaseConnectionEventChannelHandler> defaultConnHandlerList = new ArrayList<>();
    private final List<BaseConnectionEventChannelHandler> connLazyHandlerList = new ArrayList<>();
    private final BinaryMessenger binaryMgr;

    public ConnEveChlRegister(BinaryMessenger binaryMessenger) {
        this.binaryMgr = binaryMessenger;
    }

    public void register() {
        registerEveChl(new ConnectionStateHandler(), EVENT_CONNECTION_STATE);
        registerEveChl(new StepsChangeHandler(), EVENT_STEPS_CHANGE);
        registerEveChl(new DeviceBatteryHandler(), EVENT_DEVICE_BATTERY);
        registerEveChl(new WeatherChangeHandler(), EVENT_WEATHER_CHANGE);
        registerEveChl(new StepsDetailHandler(), EVENT_STEPS_DETAIL);
        registerEveChl(new SleepChangeHandler(), EVENT_SLEEP_CHANGE);
        registerLazyEveChl(new FileFirmwareUpgradeHandler(), EVENT_FIRMWARE_UPGRADE);
        registerEveChl(new HeartRateHandler(), EVENT_HEART_RATE);
        registerEveChl(new BloodPressureHandler(), EVENT_BLOOD_PRESSURE);
        registerEveChl(new BloodOxygenHandler(), EVENT_BLOOD_OXYGEN);
        registerEveChl(new CameraHandler(), EVENT_CAMERA);
        registerEveChl(new PhoneHandler(), EVENT_PHONE);
        registerEveChl(new DeviceRssiHandler(), EVENT_DEVICE_RSSI);
        registerLazyEveChl(new FileTransHandler(), EVENT_FILE_TRANS);
        registerLazyEveChl(new WFFileTransHandler(), EVENT_WF_FILE_TRANS);
        registerLazyEveChl(new ECGHandler(), EVENT_ECG);
        registerLazyEveChl(new ContactAvatarHandler(), EVENT_CONTACT_AVATAR);
        registerLazyEveChl(new CallNumberHandler(), EVENT_CALL_NUMBER);
        registerLazyEveChl(new FindPhoneHandler(), EVENT_FIND_PHONE);
        registerEveChl(new TrainingStateHandler(), EVENT_TRAINING_STATE);
        registerEveChl(new TempChangeHandler(), EVENT_TEMP_CHANGE);
        registerEveChl(new ContactHandler(), EVENT_CONTACT);
        registerEveChl(new BatterySavingHandler(), EVENT_BATTERY_SAVING);
        registerEveChl(new TrainHandler(), EVENT_TRAIN);
        registerEveChl(new SosHandler(), EVENT_SOS_CHANGE);
        registerEveChl(new HRVHandler(), EVENT_NEW_HRV);
        registerEveChl(new StressHandler(), EVENT_STRESS);
        registerEveChl(new CalendarEventHandler(), EVENT_CALENDAR_EVENT);
        registerEveChl(new GpsChangeEventHandler(), EVENT_GPS_CHANGE_EVENT);
    }

    public void unregister() {
        for (EventChannel eventChannel : eveChlList) {
            eventChannel.setStreamHandler(null);
        }
    }

    public void bindActivity(Activity activity) {
        for (BaseConnectionEventChannelHandler chlHandler : defaultConnHandlerList) {
            chlHandler.bindActivity(activity);
        }
        for (BaseConnectionEventChannelHandler chlHandler : connLazyHandlerList) {
            chlHandler.bindActivity(activity);
        }
    }

    public BaseConnectionEventChannelHandler getEveChlHandler(String chlName) {
        for (BaseConnectionEventChannelHandler baseConnEveChlHandler : defaultConnHandlerList) {
            if (chlName.equals(baseConnEveChlHandler.getBelongChl())) {
                return baseConnEveChlHandler;
            }
        }

        return null;
    }

    public BaseConnectionEventChannelHandler getLazyEveChlHandler(String chlName) {
        for (BaseConnectionEventChannelHandler baseConnLazyEveChlHandler : connLazyHandlerList) {
            if (chlName.equals(baseConnLazyEveChlHandler.getBelongChl())) {
                return baseConnLazyEveChlHandler;
            }
        }

        return null;
    }

    private void registerEveChl(BaseConnectionEventChannelHandler eveChlHandler, String chlName) {
        EventChannel bleConnectEveChl = new EventChannel(binaryMgr, chlName);
        bleConnectEveChl.setStreamHandler(eveChlHandler);

        eveChlHandler.setBelongChl(chlName);

        eveChlList.add(bleConnectEveChl);
        defaultConnHandlerList.add(eveChlHandler);
    }

    private void registerLazyEveChl(BaseConnectionEventChannelHandler lazyEveChlHandler, String chlName) {
        EventChannel bleConnectEveChl = new EventChannel(binaryMgr, chlName);
        bleConnectEveChl.setStreamHandler(lazyEveChlHandler);

        lazyEveChlHandler.setBelongChl(chlName);

        eveChlList.add(bleConnectEveChl);
        connLazyHandlerList.add(lazyEveChlHandler);
    }
}
