package com.moyoung.moyoung_ble_plugin.event;

import android.util.Log;

import com.crrepa.ble.conn.CRPBleConnection;
import com.crrepa.ble.conn.bean.CRPWatchFaceBackgroundInfo;
import com.crrepa.ble.conn.listener.CRPFileTransListener;
import com.moyoung.moyoung_ble_plugin.ErrorConstant;
import com.moyoung.moyoung_ble_plugin.base.BaseConnectionEventChannelHandler;
import com.moyoung.moyoung_ble_plugin.common.GsonUtils;
import com.moyoung.moyoung_ble_plugin.model.FileTransBean;
import com.moyoung.moyoung_ble_plugin.model.WatchFaceBgBean;

import io.flutter.plugin.common.MethodCall;

class FileTransHandler extends BaseConnectionEventChannelHandler {

    @Override
    public void setConnListener(CRPBleConnection connection, MethodCall call) {
        String jsonStr = call.argument("watchFaceBackgroundInfo");
        WatchFaceBgBean watchFaceBgBean = GsonUtils.get().fromJson(jsonStr, WatchFaceBgBean.class);

        boolean bitmapFlag = watchFaceBgBean.getBitmap().getWidth() == watchFaceBgBean.getWidth() &&
                watchFaceBgBean.getBitmap().getHeight() == watchFaceBgBean.getHeight();
        boolean thumbBitmapFlag = watchFaceBgBean.getThumbBitmap().getWidth() == watchFaceBgBean.getThumbWidth() &&
                watchFaceBgBean.getThumbBitmap().getHeight() == watchFaceBgBean.getThumbHeight();

        if (bitmapFlag && thumbBitmapFlag) {
            CRPWatchFaceBackgroundInfo backgroundInfo = new CRPWatchFaceBackgroundInfo(watchFaceBgBean.getBitmap(), watchFaceBgBean.getThumbBitmap(), watchFaceBgBean.getType());

            sendWatchFaceBackground(connection, backgroundInfo);
        } else {
            if (eventSink != null) {
                eventSink.error(ErrorConstant.IMAGE_ADAPTATION_ERROR,
                        "Image width and width do not match",
                        "The picture does not fit");
            }
        }
    }

    private void sendWatchFaceBackground(CRPBleConnection connection, CRPWatchFaceBackgroundInfo backgroundInfo) {
        connection.sendWatchFaceBackground(backgroundInfo, new CRPFileTransListener() {
            @Override
            public void onTransProgressStarting() {
                if (eventSink == null) {
                    return;
                }
                FileTransBean transBean = new FileTransBean(FileTransBean.TRANSSTART);
                String bgProgress = GsonUtils.get().toJson(transBean, FileTransBean.class);
                activity.runOnUiThread(() -> eventSink.success(bgProgress));
            }

            @Override
            public void onTransProgressChanged(int progress) {
                if (eventSink == null) {
                    return;
                }
                FileTransBean transBean = new FileTransBean(FileTransBean.TRANSCHANGED);
                transBean.setProgress(progress);
                String bgProgress = GsonUtils.get().toJson(transBean, FileTransBean.class);
                activity.runOnUiThread(() -> eventSink.success(bgProgress));
            }

            @Override
            public void onTransCompleted() {
                if (eventSink == null) {
                    return;
                }
                FileTransBean transBean = new FileTransBean(FileTransBean.TRANSCOMPLETED);
                transBean.setProgress(100);
                String bgProgress = GsonUtils.get().toJson(transBean, FileTransBean.class);
                activity.runOnUiThread(() -> eventSink.success(bgProgress));
            }

            @Override
            public void onError(int i) {
                if (eventSink == null) {
                    return;
                }
                FileTransBean transBean = new FileTransBean(FileTransBean.ERROR);
                transBean.setError(i);
                transBean.setProgress(-1);
                String bgProgress = GsonUtils.get().toJson(transBean, FileTransBean.class);
                activity.runOnUiThread(() -> eventSink.success(bgProgress));
            }
        });
    }
}

