package com.moyoung.moyoung_ble_plugin.method;

import static com.moyoung.moyoung_ble_plugin.ChannelNames.EVENT_FIRMWARE_UPGRADE;

import com.crrepa.ble.conn.CRPBleConnection;
import com.crrepa.ble.ota.hs.HsDfuController;
import com.moyoung.moyoung_ble_plugin.event.ConnEveChlRegister;
import com.moyoung.moyoung_ble_plugin.event.FileFirmwareUpgradeHandler;

import io.flutter.plugin.common.MethodCall;

public class OTAController {
    private int type;
    private FileFirmwareUpgradeHandler handler;
    private MethodCall call;
    private CRPBleConnection bleConnection;
    public static final int START_HS_OTA = 1;
    public static final int START_RTK_OTA = 2;
    public static final int START_OTA = 3;
    public static final int START_DEFAULT_OTA = 4;

    public OTAController(int type, ConnEveChlRegister connEveChlRegister, MethodCall call, CRPBleConnection bleConnection) {
        this.type = type;
        this.handler = (FileFirmwareUpgradeHandler) connEveChlRegister.getLazyEveChlHandler(EVENT_FIRMWARE_UPGRADE);
        this.call = call;
        this.bleConnection = bleConnection;
    }

    public OTAController() {
    }

    public OTAController(int type) {
        this.type = type;
    }

    public void startOTA() {
        if (this.handler != null) {

            switch (this.type) {
                case START_HS_OTA:
                    this.handler.setHsLazyConnListener(call);
                case START_RTK_OTA:
                    this.handler.setRtkUpgradeListener(call);
                default:
                    this.handler.setConnListener(bleConnection, call);
            }
        }
    }

    public void abortOTA() {
        if (this.handler != null) {

            switch (this.type) {
                case START_HS_OTA:
                    HsDfuController hsDfuController = HsDfuController.getInstance();
                    hsDfuController.abort();
                default:
                    bleConnection.abortFirmwareUpgrade();
            }
        }
    }
}
