import 'dart:async';

import 'package:moyoung_ble_plugin/impl/moyoung_beans.dart';
import 'package:moyoung_ble_plugin/impl/moyoung_ble_impl.dart';

// Export [moyoung_beans], [moyoung_contants] from the moyoung_ble
// so plugin users can use them directly.
export 'package:moyoung_ble_plugin/impl/moyoung_beans.dart';
export 'package:moyoung_ble_plugin/impl/moyoung_contants.dart';

class MoYoungBle {
  /// Constructs a singleton instance of [MoYoungBle].
  ///
  /// [MoYoungBle] is designed to work as a singleton.
  // factory MoYoungBle() {
  //   _singleton ??= MoYoungBle._();
  //   return _singleton!;
  // }
  //
  // MoYoungBle._();
  //
  // static MoYoungBle? _singleton;

  // static MoyoungBleImpl get MoyoungBleImpl() {
  //   return MoyoungBleImpl.instance;
  // }
  

  /// Clicking the scan button will trigger the scan event listener stream
  /// bleScanEveStm, and the scan result will be returned through the
  /// bleScanEveStm listener stream and stored in the event as a BleScanEvent
  /// object.
  Stream<BleScanBean> get bleScanEveStm {
    return MoyoungBleImpl().bleScanEveStm;
  }

  /// When scanning Bluetooth devices, it will first obtain the open status of
  /// Bluetooth, and only when the  permission is allowed and the Bluetooth is
  /// turned on can start normal scanning.
  Future<bool> startScan(int scanPeriod) {
    return MoyoungBleImpl().startScan(scanPeriod);
  }

  /// Turn off the bluetooth scan, and the result listens to the callback
  /// through the bleScanEveStm data stream.
  Future<void> get cancelScan {
    return MoyoungBleImpl().cancelScan;
  }

  /// Gets the watch's Mac address by scanning the received CRPScanDevice. Click
  /// the Bluetooth connection to connect the device, and the Bluetooth
  /// connection monitoring will be triggered at the same time. Monitor
  /// connection and callback status via the connStateEveStm data stream. The
  /// result is saved in "event". It is recommended to add an appropriate delay
  /// when disconnecting and reconnecting, so that the system can recover
  /// resources and ensure the connection success rate.
  Stream<dynamic> get connStateEveStm {
    return MoyoungBleImpl().connStateEveStm;
  }

  /// Gets the current status of the watch's successful connection according to
  /// the watch's mac address.
  Future<bool> isConnected(String address) {
    return MoyoungBleImpl().isConnected(address);
  }

  /// Gets the watch's Mac address by scanning the received CRPScanDevice.
  /// Click the Bluetooth connection to connect the device, and the Bluetooth
  /// connection monitoring will be triggered at the same time. Monitor
  /// connection and callback status via the connStateEveStm data stream.
  /// The result is saved in "event". It is recommended to add an appropriate
  /// delay when disconnecting and reconnecting, so that the system can recover
  /// resources and ensure the connection success rate.
  Future<void> connect(String address) {
    return MoyoungBleImpl().connect(address);
  }

  /// Disconnected watch,disconnect returns true.
  Future<bool> get disconnect {
    return MoyoungBleImpl().disconnect();
  }

  Future<bool> get closeGatt {
    return MoyoungBleImpl().closeGatt();
  }

  Future<void> get reconnect {
    return MoyoungBleImpl().reconnect();
  }

  Future<void> connectDevice(ConnectDeviceBean connectDeviceBean) {
    return MoyoungBleImpl().connectDevice(connectDeviceBean);
  }

  /// Synchronize the time of your phone and watch.
  Future<void> get queryTime {
    return MoyoungBleImpl().queryTime;
  }

  /// Sets the system time of the watch.
  Future<void> sendTimeSystem(int timeSystemType) {
    return MoyoungBleImpl().sendTimeSystem(timeSystemType);
  }

  /// Gets the time system of the watch.
  Future<int> get queryTimeSystem {
    return MoyoungBleImpl().queryTimeSystem;
  }

  /// Gets the current firmware version of the watch.
  Future<String> get queryFirmwareVersion {
    return MoyoungBleImpl().queryFirmwareVersion;
  }

  /// Gets the latest version information.
  Future<CheckFirmwareVersionBean> checkFirmwareVersion(FirmwareVersion info) {
    return MoyoungBleImpl().checkFirmwareVersion(info);
  }

  /// Monitoring of firmware version updates。
  Stream<OTABean> get oTAEveStm {
    return MoyoungBleImpl().oTAEveStm;
  }

  /// The way to upgrade
  Future<void> startOTA(OtaBean otaBean) {
    return MoyoungBleImpl().startOTA(otaBean);
  }

  ///Abort method
  Future<void> abortOTA(int oTAType) async {
    return MoyoungBleImpl().abortOTA(oTAType);
  }

  /// Watch OTA status.
  Future<int> get queryDeviceOtaStatus {
    return MoyoungBleImpl().queryDeviceOtaStatus;
  }

  Future<String> get queryHsOtaAddress {
    return MoyoungBleImpl().queryHsOtaAddress;
  }

  Future<void> get enableHsOta {
    return MoyoungBleImpl().enableHsOta;
  }

  Future<int> get queryOtaType {
    return MoyoungBleImpl().queryOtaType;
  }

  /// Sets the watch battery monitoring stream connDeviceBatteryEveStm to return
  /// data about the watch battery.
  Stream<DeviceBatteryBean> get deviceBatteryEveStm {
    return MoyoungBleImpl().deviceBatteryEveStm;
  }

  /// Gets the current battery of the watch. When the battery level of the watch
  /// exceeds 100, it means the watch is charging.
  Future<void> get queryDeviceBattery {
    return MoyoungBleImpl().queryDeviceBattery;
  }

  /// When the battery of the watch changes, the result is returned through the
  /// data stream deviceBatteryEveStmand saved in the subscribe field in "event".
  Future<void> get subscribeDeviceBattery {
    return MoyoungBleImpl().subscribeDeviceBattery;
  }

  /// Sets the user's personal information to the watch.
  Future<void> sendUserInfo(UserBean userInfo) {
    return MoyoungBleImpl().sendUserInfo(userInfo);
  }

  /// In the watch firmware 1.6.6 and above, you can set the step length to the
  /// watch to calculate the activity data more accurately.
  Future<void> sendStepLength(int stepLength) {
    return MoyoungBleImpl().sendStepLength(stepLength);
  }

  /// Sets up a weather listener weatherChangeEveStm, and the update result of
  /// the weather state is returned through the data stream and saved in the "event".
  Stream<WeatherChangeBean> get weatherChangeEveStm {
    return MoyoungBleImpl().weatherChangeEveStm;
  }

  /// Set the watch's weather for today.
  Future<void> sendTodayWeather(TodayWeatherBean todayWeatherInfo) {
    return MoyoungBleImpl().sendTodayWeather(todayWeatherInfo);
  }

  /// Sets the weather for the next 7 days to the watch.
  Future<void> sendFutureWeather(FutureWeatherListBean futureWeathersBean) {
    return MoyoungBleImpl().sendFutureWeather(futureWeathersBean);
  }

  /// Sets a step change listener stepChangeEveStm, the result is returned
  /// through the data stream, and saved in "event" as the StepChangeBean object.
  Stream<StepsChangeBean> get stepsChangeEveStm {
    return MoyoungBleImpl().stepsChangeEveStm;
  }

  /// Gets today's step count data. The query result will be obtained through
  /// the stepsChangeEveStm listening stream and saved in the
  /// StepChangeBean.stepInfo field.
  Future<void> get querySteps {
    return MoyoungBleImpl().querySteps;
  }

  /// 1.The watch can save the number of activity steps in the past three days,
  /// and can query the number of activity steps in a certain day.
  /// 2.Query the activity steps data in a certain day. The query result will be
  /// obtained through the trainingEveStm listening stream and saved in the
  /// trainingInfo.past field and trainingInfo.pastStepInfo field.
  Future<void> queryHistorySteps(int historyTimeType) {
    return MoyoungBleImpl().queryHistorySteps(historyTimeType);
  }

  /// Some watches support categorical statistics for the past two days.
  /// Query classification statistics for the past two days. The query result
  /// will be obtained through the stepsDetailEveStm listening stream and saved
  /// in "event" as the StepsDetailBean object.
  Future<void> queryStepsDetail(int stepsDetailDateType) {
    return MoyoungBleImpl().queryStepsDetail(stepsDetailDateType);
  }

  Future<The24HourListBean> get get24HourCals {
    return MoyoungBleImpl().get24HourCals;
  }

  Future<The24HourListBean> get getAgo24HourCals {
    return MoyoungBleImpl().getAgo24HourCals;
  }

  Future<The24HourListBean> get get24HourDistances {
    return MoyoungBleImpl().get24HourDistances;
  }

  Future<The24HourListBean> get getAgo24HourDistances {
    return MoyoungBleImpl().getAgo24HourDistances;
  }

  Future<void> queryActionDetails(int actionDetailsType) {
    return MoyoungBleImpl().queryActionDetails(actionDetailsType);
  }

  /// Set a step category listener stepsDetailEveStm, and the result is returned
  /// through the data stream and saved in the "event" as the StepsDetailBean object.
  Stream<StepsDetailBean> get stepsDetailEveStm {
    return MoyoungBleImpl().stepsDetailEveStm;
  }

  /// Set up a sleep monitor sleepChangeEveStm, and save the returned value
  /// in "event" with the value of the SleepBean object.
  Stream<SleepBean> get sleepChangeEveStm {
    return MoyoungBleImpl().sleepChangeEveStm;
  }

  /// Query detailed data for a training. The query result will be obtained
  /// through the sleepChangeEveStm listening stream and saved in the
  /// SleepBean.sleepInfo field
  Future<void> get querySleep {
    return MoyoungBleImpl().querySleep;
  }

  ///  added rem sleep state
  Future<void> get queryRemSleep {
    return MoyoungBleImpl().queryRemSleep;
  }

  Future<void> sendGoalSleepTime(int minutes) {
    return MoyoungBleImpl().sendGoalSleepTime(minutes);
  }

  Future<void> get queryGoalSleepTime {
    return MoyoungBleImpl().queryGoalSleepTime;
  }

  /// Gets the sleep data of a certain day. The query result will be obtained
  /// through the sleepChangeEveStm listening stream and saved in the
  /// SleepBean.past field and the SleepBean.pastSleepInfo field.
  Future<void> queryHistorySleep(int historyTimeType) {
    return MoyoungBleImpl().queryHistorySleep(historyTimeType);
  }

  /// Gets the unit system.
  Future<int> get queryUnitSystem {
    return MoyoungBleImpl().queryUnitSystem;
  }

  /// The watch supports setting the time system to metric and imperial
  Future<void> sendUnitSystem(int unitSystemType) {
    return MoyoungBleImpl().sendUnitSystem(unitSystemType);
  }

  /// Turns the quick view on or off.
  Future<void> sendQuickView(bool quickViewState) {
    return MoyoungBleImpl().sendQuickView(quickViewState);
  }

  /// Gets the quick view state of the device.
  Future<bool> get queryQuickView {
    return MoyoungBleImpl().queryQuickView;
  }

  /// The watch supports setting the effective time period for turning the wrist
  /// and turning on the screen, and it is only valid when turning the wrist and
  /// turning on the screen within the set time period.
  Future<void> sendQuickViewTime(PeriodTimeBean periodTimeInfo) {
    return MoyoungBleImpl().sendQuickViewTime(periodTimeInfo);
  }

  /// Gets the effective time for quick view
  Future<PeriodTimeResultBean> get queryQuickViewTime {
    return MoyoungBleImpl().queryQuickViewTime;
  }

  /// Push the user's target step number to the watch. When the number of
  /// activity steps on the day reaches the target number of steps, the watch
  /// will remind you to reach the target.
  Future<void> sendGoalSteps(int goalSteps) {
    return MoyoungBleImpl().sendGoalSteps(goalSteps);
  }

  /// Gets the target number of steps set in the watch.
  Future<int> get queryGoalSteps {
    return MoyoungBleImpl().queryGoalSteps;
  }

  /// Daily goals are to set daily target values.
  Future<void> sendDailyGoals(DailyGoalsInfoBean dailyGoals) {
    return MoyoungBleImpl().sendDailyGoals(dailyGoals);
  }

  /// Querying Daily Goals.
  Future<DailyGoalsInfoBean> get queryDailyGoals {
    return MoyoungBleImpl().queryDailyGoals;
  }

  /// Set workout day goals.
  Future<void> sendTrainingDayGoals(DailyGoalsInfoBean dailyGoals) {
    return MoyoungBleImpl().sendTrainingDayGoals(dailyGoals);
  }

  /// Look up workout day goals.
  Future<DailyGoalsInfoBean> get queryTrainingDayGoals {
    return MoyoungBleImpl().queryTrainingDayGoals;
  }

  /// Set up a workout Day.
  Future<void> sendTrainingDays(TrainingDayInfoBean trainingDay) {
    return MoyoungBleImpl().sendTrainingDays(trainingDay);
  }

  /// Check Exercise Day.
  Future<TrainingDayInfoBean> get queryTrainingDay {
    return MoyoungBleImpl().queryTrainingDay;
  }

  /// The watch supports a variety of different watchfaces, which can be switched freely.
  /// Send watchface type,Parameters provided by WatchFaceType.
  Future<void> sendDisplayWatchFace(int watchFaceType) {
    return MoyoungBleImpl().sendDisplayWatchFace(watchFaceType);
  }

  /// Gets the type that the watch face is currently using.
  Future<int> get queryDisplayWatchFace {
    return MoyoungBleImpl().queryDisplayWatchFace;
  }

  /// Gets the watchface layout.
  Future<WatchFaceLayoutBean> get queryWatchFaceLayout {
    return MoyoungBleImpl().queryWatchFaceLayout;
  }

  /// Sets the watchface layout
  Future<void> sendWatchFaceLayout(WatchFaceLayoutBean watchFaceLayoutInfo) {
    return MoyoungBleImpl().sendWatchFaceLayout(watchFaceLayoutInfo);
  }

  /// The dial of the 1.3-inch color screen supports the replacement of the
  /// background image with a picture size of 240 * 240 px. Compressed indicates
  /// whether the picture needs to be compressed (the watch with the master
  /// control of 52840 does not support compression and is fixed to false);
  /// timeout indicates the timeout period, in seconds. The progress is called
  /// back by _blePlugin.lazyFileTransEveStm.listen.
  Future<void> sendWatchFaceBackground(WatchFaceBackgroundBean watchFaceBackgroundInfo) async {
    return MoyoungBleImpl().sendWatchFaceBackground(watchFaceBackgroundInfo);
  }

  Future<void> get abortWatchFaceBackground {
    return MoyoungBleImpl().abortWatchFaceBackground;
  }

  /// Monitor the process of sending the dial background.
  Stream<FileTransBean> get fileTransEveStm {
    return MoyoungBleImpl().fileTransEveStm;
  }

  /// When the watch switches dials, it needs to query the type supported by the
  /// dial.
  Future<SupportWatchFaceBean> get querySupportWatchFace {
    return MoyoungBleImpl().querySupportWatchFace;
  }

  /// Gets the list of available watch faces by way of paging query.
  Future<List<WatchFaceBean>> queryWatchFaceStore(WatchFaceStoreBean watchFaceStoreBean) {
    return MoyoungBleImpl().queryWatchFaceStore(watchFaceStoreBean);
  }

  /// Gets the watchface information of the watchface ID.
  Future<WatchFaceIdBean> queryWatchFaceOfID(int id) {
    return MoyoungBleImpl().queryWatchFaceOfID(id);
  }

  Stream<FileTransBean> get wfFileTransEveStm {
    return MoyoungBleImpl().wfFileTransEveStm;
  }

  /// Send the watchface file of the new watchface to the watch, during which
  /// the watch will restart.
  Future<void> sendWatchFace(SendWatchFaceBean sendWatchFaceBean) async {
    return MoyoungBleImpl().sendWatchFace(sendWatchFaceBean);
  }

  Future<void> get abortWatchFace {
    return MoyoungBleImpl().abortWatchFace;
  }

  Future<void> deleteWatchFace(int id) {
    return MoyoungBleImpl().deleteWatchFace(id);
  }

  Future<int> get queryAvailableStorage {
    return MoyoungBleImpl().queryAvailableStorage;
  }

  /// The watch supports three alarm clocks, and the alarm clock information can
  /// be set according to the alarm clock serial number. Single alarm clock
  /// supports setting date.
  Future<void> sendAlarm(AlarmClockBean alarmClockInfo) {
    return MoyoungBleImpl().sendAlarm(alarmClockInfo);
  }

  /// Gets all alarm clock information saved by the watch.
  Future<List<AlarmClockBean>> get queryAllAlarm {
    return MoyoungBleImpl().queryAllAlarm;
  }

  Future<void> sendNewAlarm(AlarmClockBean alarmClockInfo) {
    return MoyoungBleImpl().sendNewAlarm(alarmClockInfo);
  }

  Future<void> deleteNewAlarm(int id) {
    return MoyoungBleImpl().deleteNewAlarm(id);
  }

  Future<void> deleteAllNewAlarm() {
    return MoyoungBleImpl().deleteAllNewAlarm();
  }

  Future<List<AlarmClockBean>> get queryAllNewAlarm {
    return MoyoungBleImpl().queryAllNewAlarm;
  }

  /// Sets blood pressure listener
  Stream<HeartRateBean> get heartRateEveStm {
    return MoyoungBleImpl().heartRateEveStm;
  }

  /// Gets the last measured heart rate record saved by the watch. The query
  /// result will be obtained through the heartRateEveStm listening stream and
  /// saved in the HeartRateBean.measuring field.
  Future<void> queryLastDynamicRate(String type) {
    return MoyoungBleImpl().queryLastDynamicRate(type);
  }

  /// The watch supports 24-hour timed measurement of heart rate, starting from
  /// 00:00, you can set the measurement interval, the time interval is a
  /// multiple of 5 minutes.
  Future<void> enableTimingMeasureHeartRate(int interval) {
    return MoyoungBleImpl().enableTimingMeasureHeartRate(interval);
  }

  /// The query timing measures the heart rate on state.
  Future<void> get disableTimingMeasureHeartRate {
    return MoyoungBleImpl().disableTimingMeasureHeartRate;
  }

  /// The query timing measures the heart rate on state.
  Future<int> get queryTimingMeasureHeartRate {
    return MoyoungBleImpl().queryTimingMeasureHeartRate;
  }

  /// Query today's measured heart rate value. The query result will be obtained
  /// through the heartRateEveStm listening stream and saved in the
  /// HeartRateBean.hour24MeasureResult field.
  Future<void> queryTodayHeartRate(int heartRateType) {
    return MoyoungBleImpl().queryTodayHeartRate(heartRateType);
  }

  /// Gets the heart rate data of the previous day. The query result will be
  /// obtained through the heartRateEveStm listening stream and saved in the
  /// HeartRateBean.hour24MeasureResult field.
  Future<void> queryPastHeartRate(int historyDay) {
    return MoyoungBleImpl().queryPastHeartRate(historyDay);
  }

  /// Some watchs support heart rate measurement in a variety of motion modes.
  /// The measurements include other motion-related data such as heart rate and
  /// calories. This interface is used to obtain data such as calories.
  /// The watch can save the last three sports data. Supporting 24-hour
  /// continuous measurement of the watch, the exercise heart rate can be
  /// obtained from the 24-hour heart rate data according to the movement up time;
  /// other watch exercise heart rate and dynamic heart rate acquisition
  /// methods are consistent.
  Future<void> get queryTrainingHeartRate {
    return MoyoungBleImpl().queryTrainingHeartRate;
  }

  /// Start measuring a single heart rate, the query result will be obtained
  /// through the heartRateEveStm listening stream and saved in the
  /// HeartRateBean.onceMeasureComplete field.
  Future<void> get startMeasureOnceHeartRate {
    return MoyoungBleImpl().startMeasureOnceHeartRate;
  }

  /// End a once measurement. A measurement time that is too short will result
  /// in no measurement data.
  Future<void> get stopMeasureOnceHeartRate {
    return MoyoungBleImpl().stopMeasureOnceHeartRate;
  }

  /// To query the historical heart rate, the query result will be obtained
  /// through the heartRateEveStm monitoring stream and saved in the
  /// HeartRateBean.historyHRList field.
  Future<void> get queryHistoryHeartRate {
    return MoyoungBleImpl().queryHistoryHeartRate;
  }

  /// Sets blood pressure listener
  Stream<BloodPressureBean> get bloodPressureEveStm {
    return MoyoungBleImpl().bloodPressureEveStm;
  }

  /// Measuring blood pressure
  Future<void> get startMeasureBloodPressure {
    return MoyoungBleImpl().startMeasureBloodPressure;
  }

  /// Stop measuring blood pressure, too short a measurement time will result in
  /// no measurement results. The measurement results are monitored through the
  /// bloodPressureEveStm data stream, and the returned high and low pressure
  /// values are stored in BloodPressureBean.bloodPressureChange and
  /// BloodOxygenBean.bloodPressureChange1, respectively.
  Future<void> get stopMeasureBloodPressure {
    return MoyoungBleImpl().stopMeasureBloodPressure;
  }

  /// Enable continue blood pressure
  Future<void> get enableContinueBloodPressure {
    return MoyoungBleImpl().enableContinueBloodPressure;
  }

  /// Disable continue blood pressure
  Future<void> get disableContinueBloodPressure {
    return MoyoungBleImpl().disableContinueBloodPressure;
  }

  /// The measurement results are monitored through the bloodPressureEveStmdata
  /// stream, and the value is stored in BloodPressureBean.continueState.
  Future<void> get queryContinueBloodPressureState {
    return MoyoungBleImpl().queryContinueBloodPressureState;
  }

  /// The measurement results are monitored through the bloodPressureEveStmdata
  /// stream, and the value is stored in BloodPressureBean.historyBPList.
  Future<void> get queryLast24HourBloodPressure {
    return MoyoungBleImpl().queryLast24HourBloodPressure;
  }

  /// The measurement results are monitored through the bloodPressureEveStmdata
  /// stream, and the value is stored in BloodPressureBean.historyBPList.
  Future<void> get queryHistoryBloodPressure {
    return MoyoungBleImpl().queryHistoryBloodPressure;
  }

  /// Sets blood oxygen listener.
  Stream<BloodOxygenBean> get bloodOxygenEveStm {
    return MoyoungBleImpl().bloodOxygenEveStm;
  }

  /// Measuring blood oxygen.
  Future<void> get startMeasureBloodOxygen {
    return MoyoungBleImpl().startMeasureBloodOxygen;
  }

  /// When the blood oxygen measurement is stopped, if the measurement time is
  /// too short, there will be no measurement results. The measurement results
  /// are monitored through the bloodOxygenEveStm data stream, and the value is
  /// stored in BloodOxygenBean.bloodOxygen.
  Future<void> get stopMeasureBloodOxygen {
    return MoyoungBleImpl().stopMeasureBloodOxygen;
  }

  /// measure period = interval * 5 (mins).
  Future<void> enableTimingMeasureBloodOxygen(int interval) {
    return MoyoungBleImpl().enableTimingMeasureBloodOxygen(interval);
  }

  /// Disable timing measure blood oxygen.
  Future<void> get disableTimingMeasureBloodOxygen {
    return MoyoungBleImpl().disableTimingMeasureBloodOxygen;
  }

  /// The measurement results are monitored through the bloodOxygenEveStm data
  /// stream, and the value is stored in BloodOxygenBean.timingMeasure.
  Future<void> get queryTimingBloodOxygenMeasureState {
    return MoyoungBleImpl().queryTimingBloodOxygenMeasureState;
  }

  /// The measurement results are monitored through the bloodOxygenEveStm data
  /// stream, and the value is stored in BloodOxygenBean.continueBO.
  Future<void> queryTimingBloodOxygen(String bloodOxygenTimeType) {
    return MoyoungBleImpl().queryTimingBloodOxygen(bloodOxygenTimeType);
  }

  /// Enable continue blood oxygen.
  Future<void> get enableContinueBloodOxygen {
    return MoyoungBleImpl().enableContinueBloodOxygen;
  }

  /// Disable continue blood oxygen.
  Future<void> get disableContinueBloodOxygen {
    return MoyoungBleImpl().disableContinueBloodOxygen;
  }

  ///  The measurement results are monitored through the bloodOxygenEveStm data
  ///  stream, and the value is stored in BloodOxygenBean.continueState.
  Future<void> get queryContinueBloodOxygenState {
    return MoyoungBleImpl().queryContinueBloodOxygenState;
  }

  ///  The measurement results are monitored through the bloodOxygenEveStm data
  ///  stream, and the value is stored in BloodOxygenBean.continueBO.
  Future<void> get queryLast24HourBloodOxygen {
    return MoyoungBleImpl().queryLast24HourBloodOxygen;
  }

  /// The measurement results are monitored through the bloodOxygenEveStm data
  /// stream, and the value is stored in BloodOxygenBean.historyList.
  Future<void> get queryHistoryBloodOxygen {
    return MoyoungBleImpl().queryHistoryBloodOxygen;
  }

  /// Enable camera view.
  Future<void> get enterCameraView {
    return MoyoungBleImpl().enterCameraView;
  }

  Future<void> sendDelayTaking(int delayTime) {
    return MoyoungBleImpl().sendDelayTaking(delayTime);
  }

  // Future<void> get queryDelayTaking {
  //   return MoyoungBleImpl().queryDelayTaking;
  // }

  /// Exit camera view.
  Future<void> get exitCameraView {
    return MoyoungBleImpl().exitCameraView;
  }

  /// Long press the watch photo interface to trigger the camera's camera command.
  Stream<CameraBean> get cameraEveStm {
    return MoyoungBleImpl().cameraEveStm;
  }

  /// Sets phone listener
  Stream<int> get phoneEveStm {
    return MoyoungBleImpl().phoneEveStm;
  }

  /// Sets RSSI listening
  Stream<int> get deviceRssiEveStm {
    return MoyoungBleImpl().deviceRssiEveStm;
  }

  /// Read the real-time RSSI value of the watch. The query result will be
  /// obtained through the deviceRssiEveStm listening stream and saved in
  /// the "event" field.
  Future<void> get readDeviceRssi {
    return MoyoungBleImpl().readDeviceRssi;
  }

  /// Add ECG listening event.
  Future<void> setECGChangeListener(String ecgMeasureType) {
    return MoyoungBleImpl().setECGChangeListener(ecgMeasureType);
  }

  /// Sets the ECG monitor and get the return value through connLazyEgcEveStm.
  Stream<EcgBean> get ecgEveStm {
    return MoyoungBleImpl().ecgEveStm;
  }

  /// Start to measure the ECG, the ECG measurement time is 30 seconds, and the
  /// user needs to touch the left and right electrodes of the watch with both
  /// hands. The value is obtained by listening to the lazyEcgEveStm data stream,
  /// and the value is saved in EgcBean.ints
  Future<void> get startECGMeasure {
    return MoyoungBleImpl().startECGMeasure;
  }

  /// Stop measuring ECG.
  Future<void> get stopECGMeasure {
    return MoyoungBleImpl().stopECGMeasure;
  }

  /// In the new measurement mode, the watch can save the last unsent measurement
  /// result; the old version does not.
  Future<bool> get isNewECGMeasurementVersion {
    return MoyoungBleImpl().isNewECGMeasurementVersion;
  }

  /// Gets the ECG data saved by the watch, monitor the data stream through
  /// lazyEcgEveStm, and save the value in EgcBean.ints
  Future<void> get queryLastMeasureECGData {
    return MoyoungBleImpl().queryLastMeasureECGData;
  }

  /// Using the measured data, the instantaneous heart rate is calculated
  /// through the ECG algorithm library and sent to the watch.
  Future<void> sendECGHeartRate(int heartRate) {
    return MoyoungBleImpl().sendECGHeartRate(heartRate);
  }

  /// Sets the language of the watch. When setting the language, the language
  /// version will be set. Simplified Chinese is set to the Chinese version,
  /// and non-simplified Chinese is set to the international version.
  Future<void> sendDeviceLanguage(int language) {
    return MoyoungBleImpl().sendDeviceLanguage(language);
  }

  Future<int> get queryDeviceVersion {
    return MoyoungBleImpl().queryDeviceVersion;
  }

  /// Gets the language that the watch is using and the list of languages
  /// supported by the watch.
  Future<DeviceLanguageBean> get queryDeviceLanguage {
    return MoyoungBleImpl().queryDeviceLanguage;
  }

  Future<void> sendOtherMessageState(bool state) {
    return MoyoungBleImpl().sendOtherMessageState(state);
  }

  Future<bool> get queryOtherMessageState {
    return MoyoungBleImpl().queryOtherMessageState;
  }

  /// Send various types of message content to the watch.
  Future<void> sendMessage(MessageBean messageInfo) {
    return MoyoungBleImpl().sendMessage(messageInfo);
  }

  /// When the watch receives a push of a phone type message, the watch will
  /// vibrate for a fixed time. Call this interface to stop the watch from
  /// vibrating when the watch answers the call or hangs up the call.
  Future<void> get endCall {
    return MoyoungBleImpl().endCall;
  }

  Future<void> sendCallContactName(String name) {
    return MoyoungBleImpl().sendCallContactName(name);
  }

  Future<List<int>> get queryMessageList {
    return MoyoungBleImpl().queryMessageList;
  }

  Stream<String> get callNumberEveStm {
    return MoyoungBleImpl().callNumberEveStm;
  }

  ///Sets up other push notifications<Only ios support>
  Future<void> setNotification(List<int> list) {
    return MoyoungBleImpl().setNotification(list);
  }

  ///Gets other push notifications<Only ios support>
  Future<List<int>> get getNotification {
    return MoyoungBleImpl().getNotification;
  }

  /// Turn sedentary reminders on or off.
  Future<void> sendSedentaryReminder(bool sedentaryReminder) {
    return MoyoungBleImpl().sendSedentaryReminder(sedentaryReminder);
  }

  /// Gets sedentary reminder status
  Future<bool> get querySedentaryReminder {
    return MoyoungBleImpl().querySedentaryReminder;
  }

  /// Sets the effective period of sedentary reminder.
  Future<void> sendSedentaryReminderPeriod(SedentaryReminderPeriodBean sedentaryReminderPeriodInfo) {
    return MoyoungBleImpl().sendSedentaryReminderPeriod(sedentaryReminderPeriodInfo);
  }

  /// Gets the watch for sedentary reminder valid period.
  Future<SedentaryReminderPeriodBean> get querySedentaryReminderPeriod {
    return MoyoungBleImpl().querySedentaryReminderPeriod;
  }

  /// Find the watch, the watch will vibrate for a few seconds after
  /// receiving this command.
  Future<void> get findDevice {
    return MoyoungBleImpl().findDevice;
  }

  /// The watch is turned off.
  Future<void> get shutDown {
    return MoyoungBleImpl().shutDown;
  }

  Future<void> get reset {
    return MoyoungBleImpl().reset;
  }

  /// Check if do not disturb the time set by the watch.
  Future<PeriodTimeResultBean> get queryDoNotDisturbTime {
    return MoyoungBleImpl().queryDoNotDisturbTime;
  }

  /// The watch supports the Do Not Disturb period. Do not display message push
  /// and sedentary reminders during the time.
  Future<void> sendDoNotDisturbTime(PeriodTimeBean periodTimeInfo) {
    return MoyoungBleImpl().sendDoNotDisturbTime(periodTimeInfo);
  }

  /// Sets the breathing light.
  Future<void> sendBreathingLight(bool breathingLight) {
    return MoyoungBleImpl().sendBreathingLight(breathingLight);
  }

  /// Gets the status of the breathing light.
  Future<bool> get queryBreathingLight {
    return MoyoungBleImpl().queryBreathingLight;
  }

  /// Sets the menstrual cycle reminder
  Future<void> sendMenstrualCycle(MenstrualCycleBean info) {
    return MoyoungBleImpl().sendMenstrualCycle(info);
  }

  /// Gets physiological cycle reminder
  Future<MenstrualCycleBean> get queryMenstrualCycle {
    return MoyoungBleImpl().queryMenstrualCycle;
  }

  /// When receiving a callback to find the bracelet phone, the app vibrates
  /// and plays a ringtone reminder.
  Future<void> get startFindPhone {
    return MoyoungBleImpl().startFindPhone;
  }

  /// When the user retrieves the phone, the vibrate and ringtone reminder ends,
  /// returning to the watch with this command
  Future<void> get stopFindPhone {
    return MoyoungBleImpl().stopFindPhone;
  }

  Stream<FindPhoneBean> get findPhoneEveStm {
    return MoyoungBleImpl().findPhoneEveStm;
  }

  /// Sets player state
  Future<void> setPlayerState(int musicPlayer) {
    return MoyoungBleImpl().setPlayerState(musicPlayer);
  }

  /// Sets song name
  Future<void> sendSongTitle(String title) {
    return MoyoungBleImpl().sendSongTitle(title);
  }

  /// Sets lyrics
  Future<void> sendLyrics(String lyrics) {
    return MoyoungBleImpl().sendLyrics(lyrics);
  }

  /// Close Music Control
  Future<void> get closePlayerControl {
    return MoyoungBleImpl().closePlayerControl;
  }

  /// Sets max volume
  Future<void> sendCurrentVolume(int volume) {
    return MoyoungBleImpl().sendCurrentVolume(volume);
  }

  /// Sets Current volumed
  Future<void> sendMaxVolume(int volume) {
    return MoyoungBleImpl().sendMaxVolume(volume);
  }

  /// Enable drinking reminder.
  Future<void> enableDrinkWaterReminder(DrinkWaterPeriodBean drinkWaterPeriodInfo) {
    return MoyoungBleImpl().enableDrinkWaterReminder(drinkWaterPeriodInfo);
  }

  /// Disable water reminder.
  Future<void> get disableDrinkWaterReminder {
    return MoyoungBleImpl().disableDrinkWaterReminder;
  }

  /// Gets drinking reminder
  Future<DrinkWaterPeriodBean> get queryDrinkWaterReminderPeriod {
    return MoyoungBleImpl().queryDrinkWaterReminderPeriod;
  }

  ///  Sets the heart rate alarm value
  Future<void> setMaxHeartRate(MaxHeartRateBean maxHeartRateBean) {
    return MoyoungBleImpl().setMaxHeartRate(maxHeartRateBean);
  }

  /// Gets the status of the wristband heart rate alarm and the value of
  /// the heart rate alarm.
  Future<MaxHeartRateBean> get queryMaxHeartRate {
    return MoyoungBleImpl().queryMaxHeartRate;
  }

  /// Start training
  Future<void> startTraining(int type) {
    return MoyoungBleImpl().startTraining(type);
  }

  /// Sets training state
  Future<void> setTrainingState(int training) {
    return MoyoungBleImpl().setTrainingState(training);
  }

  /// Modify the training state on the bracelet, and obtain the current
  /// measurement state by monitoring the data stream movementStateEveStm.
  Stream<int> get trainingStateEveStm {
    return MoyoungBleImpl().trainingStateEveStm;
  }

  /// Gets the protocol version.
  Future<String> get getProtocolVersion {
    return MoyoungBleImpl().getProtocolVersion;
  }

  /// Start measuring once temperature.
  Future<void> get startMeasureTemp {
    return MoyoungBleImpl().startMeasureTemp;
  }

  /// Stop measuring once temperature.
  Future<void> get stopMeasureTemp {
    return MoyoungBleImpl().stopMeasureTemp;
  }

  /// When the chronograph measurement is turned on, the watch automatically
  /// measures the temperature every half an hour.
  Future<void> get enableTimingMeasureTemp {
    return MoyoungBleImpl().enableTimingMeasureTemp;
  }

  /// Disable timing temperature measurement.
  Future<void> get disableTimingMeasureTemp {
    return MoyoungBleImpl().disableTimingMeasureTemp;
  }

  /// Gets the timing of temperature measurement status.
  Future<String> get queryTimingMeasureTempState {
    return MoyoungBleImpl().queryTimingMeasureTempState;
  }

  /// Gets timed temperature measurement results
  Future<void> queryTimingMeasureTemp(String tempTimeType) {
    return MoyoungBleImpl().queryTimingMeasureTemp(tempTimeType);
  }

  Future<void> get enableContinueTemp {
    return MoyoungBleImpl().enableContinueTemp;
  }

  Future<void> get disableContinueTemp {
    return MoyoungBleImpl().disableContinueTemp;
  }

  Future<void> get queryContinueTempState {
    return MoyoungBleImpl().queryContinueTempState;
  }

  Future<void> get queryLast24HourTemp {
    return MoyoungBleImpl().queryLast24HourTemp;
  }

  Future<void> get sendGsensorCalibration {
    return MoyoungBleImpl().sendGsensorCalibration;
  }

  /// Sets the monitoring of body temperature measurement results to return the
  /// corresponding data of body temperature.
  Stream<TempChangeBean> get tempChangeEveStm {
    return MoyoungBleImpl().tempChangeEveStm;
  }

  /// Sets display time.
  Future<void> sendDisplayTime(int time) {
    return MoyoungBleImpl().sendDisplayTime(time);
  }

  /// Gets display time.
  Future<int> get queryDisplayTime {
    return MoyoungBleImpl().queryDisplayTime;
  }

  /// Enable hand washing reminder
  Future<void> enableHandWashingReminder(HandWashingPeriodBean info) {
    return MoyoungBleImpl().enableHandWashingReminder(info);
  }

  /// Disable hand washing reminder.
  Future<void> get disableHandWashingReminder {
    return MoyoungBleImpl().disableHandWashingReminder;
  }

  /// Gets hand washing reminder.
  Future<HandWashingPeriodBean> get queryHandWashingReminderPeriod {
    return MoyoungBleImpl().queryHandWashingReminderPeriod;
  }

  /// sets local city.
  Future<void> sendLocalCity(String city) {
    return MoyoungBleImpl().sendLocalCity(city);
  }

  /// Sets temperature system.
  Future<void> sendTempUnit(int temp) {
    return MoyoungBleImpl().sendTempUnit(temp);
  }

  /// Gets temperature system.
  Future<void> get queryTempUnit {
    return MoyoungBleImpl().queryTempUnit;
  }

  /// Sets brightness.
  Future<void> sendBrightness(int brightness) {
    return MoyoungBleImpl().sendBrightness(brightness);
  }

  /// Gets brightness.
  Future<BrightnessBean> get queryBrightness {
    return MoyoungBleImpl().queryBrightness;
  }

  /// Classic Bluetooth address.
  Future<String> get queryBtAddress {
    return MoyoungBleImpl().queryBtAddress;
  }

  /// Check support contacts.
  Future<ContactConfigBean> get checkSupportQuickContact {
    return MoyoungBleImpl().checkSupportQuickContact;
  }

  /// Gets current contacts count.
  Future<int> get queryContactCount {
    return MoyoungBleImpl().queryContactCount;
  }

  /// Sets up a contact listener, and the result is returned via the data stream
  /// contactEveStm and stored in "event".
  Stream<ContactListenBean> get contactEveStm {
    return MoyoungBleImpl().contactEveStm;
  }

  Future<bool> get queryContactNumberSymbol {
    return MoyoungBleImpl().queryContactNumberSymbol;
  }

  /// Sets monitoring of contacts with avatars
  Stream<FileTransBean> get contactAvatarEveStm {
    return MoyoungBleImpl().contactAvatarEveStm;
  }

  /// Sets the contact, the result is obtained through contactEveStm.
  Future<void> sendContact(ContactBean info) {
    return MoyoungBleImpl().sendContact(info);
  }

  /// Sets the contact avatar, the result is obtained through contactAvatarEveStm.
  Future<void> sendContactAvatar(ContactBean info) {
    return MoyoungBleImpl().sendContactAvatar(info);
  }

  /// Delete contact information based on contact index.
  Future<void> deleteContact(int index) {
    return MoyoungBleImpl().deleteContact(index);
  }

  /// Delete contact avatar information based on contact index.
  Future<void> deleteContactAvatar(int index) {
    return MoyoungBleImpl().deleteContactAvatar(index);
  }

  /// Clearing all contacts
  Future<void> clearContact() {
    return MoyoungBleImpl().clearContact();
  }

  /// Sets the battery storage listener, the result is returned through the data
  /// stream, and saved in "event".
  Stream<bool> get batterySavingEveStm {
    return MoyoungBleImpl().batterySavingEveStm;
  }

  /// Sets battery saving state.
  Future<void> sendBatterySaving(bool enable) {
    return MoyoungBleImpl().sendBatterySaving(enable);
  }

  /// The query result will be obtained through the batterySavingEveStm
  /// monitoring stream and saved in the event field.
  Future<void> get queryBatterySaving {
    return MoyoungBleImpl().queryBatterySaving;
  }

  /// Gets support pill reminder.
  Future<PillReminderCallback> get queryPillReminder {
    return MoyoungBleImpl().queryPillReminder;
  }

  /// Sets pill reminder.
  Future<void> sendPillReminder(PillReminderBean info) {
    return MoyoungBleImpl().sendPillReminder(info);
  }

  /// Delete pill reminder.
  Future<void> deletePillReminder(int id) {
    return MoyoungBleImpl().deletePillReminder(id);
  }

  /// Clear pill reminder.
  Future<void> get clearPillReminder {
    return MoyoungBleImpl().clearPillReminder;
  }

  /// Gets whether it is in the wake-up state. If the result is true, it means
  /// that it is in the awake state, otherwise, it is not awake.
  Future<bool> get queryWakeState {
    return MoyoungBleImpl().queryWakeState;
  }

  /// Sets tap to wake state.
  Future<void> sendWakeState(bool enable) {
    return MoyoungBleImpl().sendWakeState(enable);
  }

  /// Set up a training listener, the results are returned through the data
  /// stream, and saved in "event".
  Stream<TrainBean> get trainingEveStm {
    return MoyoungBleImpl().trainingEveStm;
  }

  /// Gets the training records stored in the watch. The query result will be
  /// obtained through trainingEveStm listening stream and saved in the
  /// historyTrainList field.
  Future<void> get queryHistoryTraining {
    return MoyoungBleImpl().queryHistoryTraining;
  }

  /// Gets detailed data for a training. The query result will be obtained
  /// through the trainingEveStm listening stream and saved in the
  /// trainingInfo field.
  Future<void> queryTraining(int id) {
    return MoyoungBleImpl().queryTraining(id);
  }

  Stream<dynamic> get sosChangeEveStm {
    return MoyoungBleImpl().sosChangeEveStm;
  }

  Future<int> createBond(List<int> keys) {
    return MoyoungBleImpl().createBond(keys);
  }

  Stream<HrvHandlerBean> get newHrvEveStm {
    return MoyoungBleImpl().newHrvEveStm;
  }

  Future<void> get querySupportNewHrv {
    return MoyoungBleImpl().querySupportNewHrv;
  }

  Future<void> get startMeasureNewHrv {
    return MoyoungBleImpl().startMeasureNewHrv;
  }

  Future<void> get stopMeasureNewHrv {
    return MoyoungBleImpl().stopMeasureNewHrv;
  }

  Future<void> get queryHistoryNewHrv {
    return MoyoungBleImpl().queryHistoryNewHrv;
  }

  Stream<StressHandlerBean> get stressEveStm {
    return MoyoungBleImpl().stressEveStm;
  }

  Future<void> get querySupportStress {
    return MoyoungBleImpl().querySupportStress;
  }

  Future<void> get startMeasureStress {
    return MoyoungBleImpl().startMeasureStress;
  }

  Future<void> get stopMeasureStress {
    return MoyoungBleImpl().stopMeasureStress;
  }

  Future<void> get queryHistoryStress {
    return MoyoungBleImpl().queryHistoryStress;
  }

  Future<void> get enableTimingStress {
    return MoyoungBleImpl().enableTimingStress;
  }

  Future<void> get disableTimingStress {
    return MoyoungBleImpl().disableTimingStress;
  }

  Future<void> get queryTimingStressState {
    return MoyoungBleImpl().queryTimingStressState;
  }

  Future<void> queryTimingStress(String stressDate) {
    return MoyoungBleImpl().queryTimingStress(stressDate);
  }

  Future<ElectronicCardCountInfoBean> get queryElectronicCardCount {
    return MoyoungBleImpl().queryElectronicCardCount;
  }

  Future<void> sendElectronicCard(ElectronicCardInfoBean electronicCardInfo) {
    return MoyoungBleImpl().sendElectronicCard(electronicCardInfo);
  }

  Future<void> deleteElectronicCard(int id) {
    return MoyoungBleImpl().deleteElectronicCard(id);
  }

  Future<ElectronicCardInfoBean> queryElectronicCard(int id) {
    return MoyoungBleImpl().queryElectronicCard(id);
  }

  Future<void> sendElectronicCardList(List idList) {
    return MoyoungBleImpl().sendElectronicCardList(idList);
  }

  Stream<CalendarEventBean> get calendarEventEveStem {
    return MoyoungBleImpl().calendarEventEveStem;
  }

  Future<void> get querySupportCalendarEvent async {
    await MoyoungBleImpl().querySupportCalendarEvent;
  }

  Future<void> sendCalendarEvent(CalendarEventInfoBean calenderEventInfo) async {
    await MoyoungBleImpl().sendCalendarEvent(calenderEventInfo);
  }

  Future<void> deleteCalendarEvent(int id) async {
    await MoyoungBleImpl().deleteCalendarEvent(id);
  }

  Future<void> queryCalendarEvent(int id) async {
    await MoyoungBleImpl().queryCalendarEvent(id);
  }

  Future<void> sendCalendarEventReminderTime(CalendarEventReminderTimeBean calendarEventReminderTime) async {
    await MoyoungBleImpl().sendCalendarEventReminderTime(calendarEventReminderTime);
  }

  Future<void> get queryCalendarEventReminderTime async {
    await MoyoungBleImpl().queryCalendarEventReminderTime;
  }

  Future<void> get clearCalendarEvent async {
    await MoyoungBleImpl().clearCalendarEvent;
  }

  // ignore: non_constant_identifier_names
  Stream<int> get gpsChangeEveStm {
    return MoyoungBleImpl().gpsChangeEveStm;
  }

  Future<void> get queryHistoryGps async {
    await MoyoungBleImpl().queryHistoryGps;
  }

  Future<void> queryGpsDetail(int seconds) async {
    await MoyoungBleImpl().queryGpsDetail(seconds);
  }

  Future<void> sendVibrationStrength(int value) async {
    await MoyoungBleImpl().sendVibrationStrength(value);
  }

  Future<VibrationStrength> get queryVibrationStrength {
    return MoyoungBleImpl().queryVibrationStrength;
  }
  Future<bool> uploadLocalFile(String filePath) {
    return MoyoungBleImpl().uploadLocalFile(filePath);
  }
}
